"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Emitter = void 0;
var Emitter = (function () {
    function Emitter() {
        this._fns = [];
    }
    Emitter.prototype.on = function (fn) {
        var _this = this;
        this._fns.push(fn);
        return function () { return _this.off(fn); };
    };
    Emitter.prototype.prepend = function (fn) {
        var _this = this;
        this._fns.unshift(fn);
        return function () { return _this.off(fn); };
    };
    Emitter.prototype.once = function (fn) {
        var off = this.on(function (data, acc) {
            var r = fn(data, acc);
            if (off) {
                off();
                off = null;
            }
            return r;
        });
        return off;
    };
    Emitter.prototype.off = function (fn) {
        var _fns = this._fns;
        var i = _fns.indexOf(fn);
        if (i >= 0) {
            _fns.splice(i, 1);
        }
    };
    Emitter.prototype.emit = function (data) {
        var fns = this._fns.slice();
        var len = fns.length;
        var acc = null;
        for (var i = 0; i < len; i += 1) {
            acc = fns[i](data, acc);
        }
        return acc;
    };
    Emitter.prototype.emitGet = function (data) {
        var acc = null;
        return this._fns.slice().map(function (fn) { return (acc = fn(data, acc)); });
    };
    Emitter.prototype.count = function () {
        return this._fns.length;
    };
    Emitter.prototype.clear = function () {
        this._fns.length = 0;
    };
    return Emitter;
}());
exports.Emitter = Emitter;
//# sourceMappingURL=Emitter.js.map