"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleCache = void 0;
var tslib_1 = require("tslib");
var maybeRemoveFirst_1 = require("./array/maybeRemoveFirst");
var forEachProp_1 = require("./object/forEachProp");
var SimpleCache = (function () {
    function SimpleCache(opts) {
        this.ttlIds = {};
        if (opts) {
            this.ttl = opts.ttl;
            this.maxSize = opts.maxSize;
            this.store = opts.initialStore ? tslib_1.__assign({}, opts.initialStore) : {};
            this._keys = opts.initialKeys
                ? tslib_1.__spreadArrays(opts.initialKeys) : Object.keys(this.store);
        }
        else {
            this.store = {};
            this._keys = [];
        }
    }
    Object.defineProperty(SimpleCache.prototype, "size", {
        get: function () {
            return this._keys.length;
        },
        enumerable: false,
        configurable: true
    });
    SimpleCache.prototype.get = function (key) {
        return this.store[key];
    };
    SimpleCache.prototype.has = function (key) {
        return this.store.hasOwnProperty(key);
    };
    SimpleCache.prototype.set = function (key, value, opts) {
        var _this = this;
        var _a = this, store = _a.store, ttlIds = _a.ttlIds, _keys = _a._keys;
        if (store.hasOwnProperty(key)) {
            if (_keys[_keys.length - 1] !== key) {
                maybeRemoveFirst_1.maybeRemoveFirst(_keys, key);
                _keys.push(key);
            }
        }
        else {
            maybeRemoveFirst_1.maybeRemoveFirst(_keys, key);
            _keys.push(key);
            if (this.maxSize && _keys.length > this.maxSize) {
                this.delete(_keys.shift());
            }
        }
        store[key] = value;
        var tid = ttlIds[key];
        if (tid != null) {
            delete ttlIds[key];
            clearTimeout(tid);
        }
        var ttl = opts && opts.ttl;
        if (ttl == null)
            ttl = this.ttl;
        if (ttl != null) {
            ttlIds[key] = setTimeout(function () { return _this.delete(key); }, ttl);
        }
    };
    SimpleCache.prototype.delete = function (key) {
        if (this.store.hasOwnProperty(key)) {
            delete this.store[key];
            maybeRemoveFirst_1.maybeRemoveFirst(this._keys, key);
            var tid = this.ttlIds[key];
            if (tid != null) {
                delete this.ttlIds[key];
                clearTimeout(tid);
            }
            return true;
        }
        return false;
    };
    SimpleCache.prototype.clear = function () {
        this.store = {};
        this._keys = [];
        forEachProp_1.forEachProp(this.ttlIds, function (id) { return clearTimeout(id); });
        this.ttlIds = {};
    };
    SimpleCache.prototype.forEach = function (fn) {
        var _a = this, store = _a.store, _keys = _a._keys;
        for (var i = 0, len = _keys.length; i < len; i += 1) {
            var key = _keys[i];
            fn(store[key], key);
        }
    };
    SimpleCache.prototype.getKeys = function () {
        return this._keys.slice();
    };
    SimpleCache.prototype.getValues = function () {
        var _this = this;
        return this._keys.map(function (key) { return _this.store[key]; });
    };
    return SimpleCache;
}());
exports.SimpleCache = SimpleCache;
//# sourceMappingURL=SimpleCache.js.map