"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Timestamp = void 0;
var average_1 = require("./number/average");
var padStart_1 = require("./string/padStart");
var getNumPairs = function (len) {
    var a = [];
    for (var i = 0; i < len; i += 1) {
        a.push([0, 0]);
    }
    return a;
};
var avgNano = function (a, b) {
    var n0 = a % 1000000;
    var n1 = b % 1000000;
    if (n1 < n0) {
        n1 += 1000000;
    }
    return (n1 + n0) / 2;
};
var minuteInMs = 60 * 1000;
var hourInMs = 60 * minuteInMs;
var dayInMs = 24 * hourInMs;
var Timestamp = (function () {
    function Timestamp(milliseconds, nanoseconds) {
        this.milliseconds = milliseconds;
        this.nanoseconds = nanoseconds;
    }
    Timestamp.prototype.toDate = function () {
        return new Date(this.milliseconds);
    };
    Timestamp.prototype.toMilliseconds = function () {
        return this.milliseconds + this.nanoseconds / 1000000;
    };
    Timestamp.prototype.toMillisecondsString = function () {
        return this.milliseconds + "." + padStart_1.padStart(this.nanoseconds.toString(), 6, '0');
    };
    Timestamp.prototype.toNanosecondsString = function () {
        return "" + this.milliseconds + padStart_1.padStart(this.nanoseconds.toString(), 6, '0');
    };
    Timestamp.prototype.toString = function () {
        return this.toNanosecondsString();
    };
    Timestamp.prototype.toISOString = function () {
        return this.toDate().toISOString();
    };
    Timestamp.prototype.addSeconds = function (seconds) {
        this.milliseconds += seconds * 1000;
        return this;
    };
    Timestamp.prototype.addMinutes = function (minutes) {
        this.milliseconds += minutes * minuteInMs;
        return this;
    };
    Timestamp.prototype.addHours = function (hours) {
        this.milliseconds += hours * hourInMs;
        return this;
    };
    Timestamp.prototype.addDays = function (days) {
        this.milliseconds += days * dayInMs;
        return this;
    };
    Timestamp.prototype.clone = function () {
        return new Timestamp(this.milliseconds, this.nanoseconds);
    };
    Timestamp.prototype.equal = function (ts) {
        return (ts.milliseconds === this.milliseconds &&
            ts.nanoseconds === this.nanoseconds);
    };
    Timestamp.getMilliseconds = Date.now;
    Timestamp.getNanoseconds = process && process.hrtime && (function () { return process.hrtime()[1]; });
    Timestamp.nanosecondsCalibration = 0;
    Timestamp.calibrate = function (accuracy) {
        if (accuracy === void 0) { accuracy = 100; }
        var getNanoseconds = Timestamp.getNanoseconds, getMilliseconds = Timestamp.getMilliseconds;
        var pairs = getNumPairs(accuracy);
        var len = pairs.length;
        var i = 0;
        var lastMs = getMilliseconds();
        while (i !== len) {
            var hr0 = getNanoseconds();
            var ms = getMilliseconds();
            var hr1 = getNanoseconds();
            if (ms !== lastMs) {
                lastMs = ms;
                pairs[i] = [hr0, hr1];
                i += 1;
            }
        }
        Timestamp.nanosecondsCalibration =
            Math.round(average_1.average(pairs
                .map(function (_a) {
                var a = _a[0], b = _a[1];
                return avgNano(a, b);
            })
                .sort()
                .slice(50, 450))) % 1000000;
        return Timestamp.nanosecondsCalibration;
    };
    Timestamp.currentNanoseconds = function () {
        return (Math.max(Timestamp.getNanoseconds(), 1000000) -
            Timestamp.nanosecondsCalibration) %
            1000000;
    };
    Timestamp.now = function () {
        return new Timestamp(Timestamp.getMilliseconds(), Timestamp.currentNanoseconds());
    };
    Timestamp.from = function (ms, ns) {
        if (ns === void 0) { ns = 0; }
        var t = new Date(ms).getTime();
        return t === t ? new Timestamp(t, ns) : null;
    };
    Timestamp.parseNanoseconds = function (str) {
        var ms = parseInt(str.substring(0, str.length - 6), 10);
        var ns = parseInt(str.substring(str.length - 6), 10);
        return ms === ms && ns === ns ? new Timestamp(ms, ns) : null;
    };
    Timestamp.parseMilliseconds = function (str) {
        var _a = str.split('.'), mss = _a[0], nss = _a[1];
        var ms = parseInt(mss, 10);
        var ns = parseInt(nss, 10);
        return ms === ms && ns === ns ? new Timestamp(ms, ns) : null;
    };
    return Timestamp;
}());
exports.Timestamp = Timestamp;
//# sourceMappingURL=Timestamp.js.map