"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Value = void 0;
var Value = (function () {
    function Value(_value) {
        this._value = _value;
        this.fns = [];
        this.asyncFns = [];
        this.asyncQueued = false;
    }
    Value.prototype.on = function (fn, opts) {
        var _this = this;
        (opts && opts.async ? this.asyncFns : this.fns).push(fn);
        return function () { return _this.off(fn, opts); };
    };
    Value.prototype.off = function (fn, opts) {
        var fns = opts && opts.async ? this.asyncFns : this.fns;
        var i = fns.indexOf(fn);
        if (i >= 0) {
            fns.splice(i, 1);
        }
    };
    Value.prototype.once = function (fn, opts) {
        var off = this.on(function (data, acc) {
            var r = fn(data, acc);
            if (off) {
                off();
                off = null;
            }
            return r;
        }, opts);
        return off;
    };
    Object.defineProperty(Value.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: false,
        configurable: true
    });
    Value.prototype.get = function () {
        return this._value;
    };
    Value.prototype.set = function (value) {
        var _this = this;
        var oldValue = this._value;
        var newValue = typeof value === 'function' ? value(oldValue) : value;
        if (newValue !== oldValue) {
            this._value = newValue;
            var len = this.fns.length;
            if (len > 0) {
                for (var i = 0, fns = this.fns.slice(); i < len; i += 1) {
                    fns[i](newValue, oldValue);
                }
            }
            if (!this.asyncQueued && this.asyncFns.length) {
                this.asyncQueued = true;
                setTimeout(function () {
                    _this.asyncQueued = false;
                    var newValueAsync = _this._value;
                    if (newValueAsync !== oldValue) {
                        for (var ai = 0, afns = _this.asyncFns.slice(), alen = afns.length; ai < alen; ai += 1) {
                            afns[ai](newValueAsync, oldValue);
                        }
                    }
                }, 0);
            }
        }
    };
    Value.prototype.clear = function () {
        this.fns.length = this.asyncFns.length = 0;
    };
    return Value;
}());
exports.Value = Value;
//# sourceMappingURL=Value.js.map