"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeferredPromise = exports.TimeoutError = void 0;
var tslib_1 = require("tslib");
var TimeoutError = (function (_super) {
    tslib_1.__extends(TimeoutError, _super);
    function TimeoutError() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return TimeoutError;
}(Error));
exports.TimeoutError = TimeoutError;
var DeferredPromise = (function () {
    function DeferredPromise(timeout) {
        var _this = this;
        this.timeout = timeout;
        this._resolve = null;
        this._reject = null;
        this._fulfilled = false;
        this._promise = new Promise(function (resolve, reject) {
            _this._resolve = resolve;
            _this._reject = reject;
        });
        if (timeout) {
            this._tid = setTimeout(function () {
                if (_this._reject) {
                    _this.reject(new TimeoutError('timeout'));
                }
            }, timeout);
        }
    }
    DeferredPromise.prototype.resolve = function (value) {
        var _resolve = this._resolve;
        if (_resolve) {
            this._fulfilled = true;
            this.clear();
            _resolve(value);
        }
    };
    DeferredPromise.prototype.reject = function (reason) {
        var _reject = this._reject;
        if (_reject) {
            this.clear();
            _reject(reason);
        }
    };
    DeferredPromise.prototype.then = function (onFulfilled, onRejected) {
        return this._promise.then(onFulfilled, onRejected);
    };
    DeferredPromise.prototype.catch = function (onRejected) {
        return this._promise.catch(onRejected);
    };
    DeferredPromise.prototype.finally = function (onFinally) {
        return this._promise.then(function (v) {
            onFinally();
            return v;
        }, function (e) {
            onFinally();
            throw e;
        });
    };
    DeferredPromise.prototype.clear = function () {
        this._resolve = null;
        this._reject = null;
        var _tid = this._tid;
        if (_tid) {
            clearTimeout(_tid);
            this._tid = undefined;
        }
    };
    Object.defineProperty(DeferredPromise.prototype, "promise", {
        get: function () {
            return this._promise;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeferredPromise.prototype, "pending", {
        get: function () {
            return Boolean(this._reject);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeferredPromise.prototype, "fulfilled", {
        get: function () {
            return this._fulfilled;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeferredPromise.prototype, "rejected", {
        get: function () {
            return !(this._fulfilled || this.pending);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeferredPromise.prototype, "status", {
        get: function () {
            return this.pending ? 'pending' : this.fulfilled ? 'fulfilled' : 'rejected';
        },
        enumerable: false,
        configurable: true
    });
    return DeferredPromise;
}());
exports.DeferredPromise = DeferredPromise;
//# sourceMappingURL=DeferredPromise.js.map