"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Timestamp = void 0;
const average_1 = require("./number/average");
const padStart_1 = require("./string/padStart");
const getNumPairs = (len) => {
    const a = [];
    for (let i = 0; i < len; i += 1) {
        a.push([0, 0]);
    }
    return a;
};
const avgNano = (a, b) => {
    const n0 = a % 1000000;
    let n1 = b % 1000000;
    if (n1 < n0) {
        n1 += 1000000;
    }
    return (n1 + n0) / 2;
};
const minuteInMs = 60 * 1000;
const hourInMs = 60 * minuteInMs;
const dayInMs = 24 * hourInMs;
class Timestamp {
    constructor(milliseconds, nanoseconds) {
        this.milliseconds = milliseconds;
        this.nanoseconds = nanoseconds;
    }
    toDate() {
        return new Date(this.milliseconds);
    }
    toMilliseconds() {
        return this.milliseconds + this.nanoseconds / 1000000;
    }
    toMillisecondsString() {
        return `${this.milliseconds}.${padStart_1.padStart(this.nanoseconds.toString(), 6, '0')}`;
    }
    toNanosecondsString() {
        return `${this.milliseconds}${padStart_1.padStart(this.nanoseconds.toString(), 6, '0')}`;
    }
    toString() {
        return this.toNanosecondsString();
    }
    toISOString() {
        return this.toDate().toISOString();
    }
    addSeconds(seconds) {
        this.milliseconds += seconds * 1000;
        return this;
    }
    addMinutes(minutes) {
        this.milliseconds += minutes * minuteInMs;
        return this;
    }
    addHours(hours) {
        this.milliseconds += hours * hourInMs;
        return this;
    }
    addDays(days) {
        this.milliseconds += days * dayInMs;
        return this;
    }
    clone() {
        return new Timestamp(this.milliseconds, this.nanoseconds);
    }
    equal(ts) {
        return (ts.milliseconds === this.milliseconds &&
            ts.nanoseconds === this.nanoseconds);
    }
}
exports.Timestamp = Timestamp;
Timestamp.getMilliseconds = Date.now;
Timestamp.getNanoseconds = process && process.hrtime && (() => process.hrtime()[1]);
Timestamp.nanosecondsCalibration = 0;
Timestamp.calibrate = (accuracy = 100) => {
    const { getNanoseconds, getMilliseconds } = Timestamp;
    const pairs = getNumPairs(accuracy);
    const len = pairs.length;
    let i = 0;
    let lastMs = getMilliseconds();
    while (i !== len) {
        const hr0 = getNanoseconds();
        const ms = getMilliseconds();
        const hr1 = getNanoseconds();
        if (ms !== lastMs) {
            lastMs = ms;
            pairs[i] = [hr0, hr1];
            i += 1;
        }
    }
    Timestamp.nanosecondsCalibration =
        Math.round(average_1.average(pairs
            .map(([a, b]) => avgNano(a, b))
            .sort()
            .slice(50, 450))) % 1000000;
    return Timestamp.nanosecondsCalibration;
};
Timestamp.currentNanoseconds = () => (Math.max(Timestamp.getNanoseconds(), 1000000) -
    Timestamp.nanosecondsCalibration) %
    1000000;
Timestamp.now = () => new Timestamp(Timestamp.getMilliseconds(), Timestamp.currentNanoseconds());
Timestamp.from = (ms, ns = 0) => {
    const t = new Date(ms).getTime();
    return t === t ? new Timestamp(t, ns) : null;
};
Timestamp.parseNanoseconds = (str) => {
    const ms = parseInt(str.substring(0, str.length - 6), 10);
    const ns = parseInt(str.substring(str.length - 6), 10);
    return ms === ms && ns === ns ? new Timestamp(ms, ns) : null;
};
Timestamp.parseMilliseconds = (str) => {
    const [mss, nss] = str.split('.');
    const ms = parseInt(mss, 10);
    const ns = parseInt(nss, 10);
    return ms === ms && ns === ns ? new Timestamp(ms, ns) : null;
};
//# sourceMappingURL=Timestamp.js.map