"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.equal = exports.equalObjects = void 0;
const is_1 = require("../is");
const getKeys_1 = require("../object/getKeys");
const hasOwnProp_1 = require("../object/hasOwnProp");
exports.equalObjects = (a, b, opts) => {
    if (a === b) {
        return true;
    }
    const { testKeys = true, keys, skipKeys, allowSubset = false } = opts;
    const skip = {};
    if (skipKeys) {
        const len = skipKeys.length;
        for (let i = 0; i < len; i += 1) {
            skip[skipKeys[i]] = 1;
        }
    }
    if (keys) {
        const len = keys.length;
        if (testKeys) {
            for (let i = 0; i < len; i += 1) {
                const k = keys[i];
                if (skip[k] !== 1) {
                    skip[k] = 1;
                    const hasKey = hasOwnProp_1.hasOwnProp(a, k);
                    if (hasKey !== hasOwnProp_1.hasOwnProp(b, k) ||
                        (hasKey && !exports.equal(a[k], b[k], opts))) {
                        return false;
                    }
                }
            }
        }
        else {
            for (let i = 0; i < len; i += 1) {
                const k = keys[i];
                if (skip[k] !== 1) {
                    skip[k] = 1;
                    if (!exports.equal(a[k], b[k], opts)) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    const keysA = getKeys_1.getKeys(a);
    const keysB = getKeys_1.getKeys(b);
    const lenA = keysA.length;
    const lenB = keysB.length;
    if (testKeys && !allowSubset) {
        if (!allowSubset && !skipKeys && lenA !== lenB) {
            return false;
        }
        for (let i = 0; i < lenA; i += 1) {
            const k = keysA[i];
            if (skip[k] !== 1) {
                skip[k] = 1;
                if (!hasOwnProp_1.hasOwnProp(b, k) || !exports.equal(a[k], b[k], opts)) {
                    return false;
                }
            }
        }
        if (!allowSubset) {
            for (let i = 0; i < lenB; i += 1) {
                const k = keysB[i];
                if (skip[k] !== 1) {
                    skip[k] = 1;
                    if (!hasOwnProp_1.hasOwnProp(a, k) || !exports.equal(a[k], b[k], opts)) {
                        return false;
                    }
                }
            }
        }
    }
    else {
        for (let i = 0; i < lenA; i += 1) {
            const k = keysA[i];
            if (skip[k] !== 1) {
                skip[k] = 1;
                if (!exports.equal(a[k], b[k], opts)) {
                    return false;
                }
            }
        }
        if (!allowSubset) {
            for (let i = 0; i < lenB; i += 1) {
                const k = keysB[i];
                if (skip[k] !== 1) {
                    skip[k] = 1;
                    if (!exports.equal(a[k], b[k], opts)) {
                        return false;
                    }
                }
            }
        }
    }
    return true;
};
exports.equal = (a, b, opts) => {
    if (a === b || (a !== a && b !== b)) {
        return true;
    }
    const { normalize, equalValues } = opts;
    if (normalize) {
        a = normalize(a);
        b = normalize(b);
        if (a === b || (a !== a && b !== b)) {
            return true;
        }
    }
    if (equalValues && equalValues(a, b, opts)) {
        return true;
    }
    if (!is_1.isObject(a) || !is_1.isObject(b)) {
        return false;
    }
    if (Array.isArray(a) !== Array.isArray(b)) {
        return false;
    }
    return exports.equalObjects(a, b, opts);
};
//# sourceMappingURL=equal.js.map