"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.shallowEqualObjects = void 0;
const getKeys_1 = require("../object/getKeys");
const hasOwnProp_1 = require("../object/hasOwnProp");
const shallowEqualValues_1 = require("./shallowEqualValues");
exports.shallowEqualObjects = (a, b, opts) => {
    if (a === b) {
        return true;
    }
    const { testKeys = true, keys, skipKeys } = opts;
    const skip = {};
    if (skipKeys) {
        const len = skipKeys.length;
        for (let i = 0; i < len; i += 1) {
            skip[skipKeys[i]] = 1;
        }
    }
    if (keys) {
        const len = keys.length;
        for (let i = 0; i < len; i += 1) {
            const k = keys[i];
            if (skip[k] !== 1) {
                skip[k] = 1;
                if (testKeys) {
                    const hasKey = hasOwnProp_1.hasOwnProp(a, k);
                    if (hasKey !== hasOwnProp_1.hasOwnProp(b, k)) {
                        return false;
                    }
                    if (!hasKey) {
                        continue;
                    }
                }
                if (!shallowEqualValues_1.shallowEqualValues(a[k], b[k], opts)) {
                    return false;
                }
            }
        }
        return true;
    }
    const keysA = getKeys_1.getKeys(a);
    const lenA = keysA.length;
    const keysB = getKeys_1.getKeys(b);
    const lenB = keysB.length;
    if (testKeys && !skipKeys && lenA !== lenB) {
        return false;
    }
    for (let i = 0; i < lenA; i += 1) {
        const k = keysA[i];
        if (skip[k] !== 1) {
            skip[k] = 1;
            if ((testKeys && !hasOwnProp_1.hasOwnProp(b, k)) ||
                !shallowEqualValues_1.shallowEqualValues(a[k], b[k], opts)) {
                return false;
            }
        }
    }
    for (let i = 0; i < lenB; i += 1) {
        const k = keysB[i];
        if (skip[k] !== 1) {
            skip[k] = 1;
            if ((testKeys && !hasOwnProp_1.hasOwnProp(a, k)) ||
                !shallowEqualValues_1.shallowEqualValues(a[k], b[k], opts)) {
                return false;
            }
        }
    }
    return true;
};
//# sourceMappingURL=shallowEqualObjects.js.map