import { DeferredPromise } from './promise/DeferredPromise';
export class AsyncQueue {
    constructor(fifo = true) {
        this.fifo = fifo;
        this._running = false;
        this._queue = [];
        this._promise = null;
    }
    add(job) {
        this._queue.push(job);
        this.run();
    }
    remove(job) {
        const { _queue } = this;
        const i = _queue.indexOf(job);
        if (i >= 0) {
            _queue.splice(i, 1);
            return i;
        }
        return i;
    }
    async run() {
        let count = 0;
        if (!this._running) {
            this._running = true;
            const { _queue } = this;
            while (_queue.length > 0) {
                const job = this.fifo ? _queue.shift() : _queue.pop();
                await job();
                count += 1;
            }
            this._running = false;
            if (this._promise) {
                this._promise.resolve();
                this._promise = null;
            }
        }
        return count;
    }
    running() {
        return this._running;
    }
    count() {
        return this._queue.length;
    }
    async wait() {
        if (this._running) {
            if (!this._promise) {
                this._promise = new DeferredPromise();
            }
            return this._promise.promise;
        }
    }
}
//# sourceMappingURL=AsyncQueue.js.map