export class Emitter {
    constructor() {
        this._fns = [];
    }
    on(fn) {
        this._fns.push(fn);
        return () => this.off(fn);
    }
    prepend(fn) {
        this._fns.unshift(fn);
        return () => this.off(fn);
    }
    once(fn) {
        let off = this.on((data, acc) => {
            const r = fn(data, acc);
            if (off) {
                off();
                off = null;
            }
            return r;
        });
        return off;
    }
    off(fn) {
        const { _fns } = this;
        const i = _fns.indexOf(fn);
        if (i >= 0) {
            _fns.splice(i, 1);
        }
    }
    emit(data) {
        const fns = this._fns.slice();
        const len = fns.length;
        let acc = null;
        for (let i = 0; i < len; i += 1) {
            acc = fns[i](data, acc);
        }
        return acc;
    }
    emitGet(data) {
        let acc = null;
        return this._fns.slice().map((fn) => (acc = fn(data, acc)));
    }
    count() {
        return this._fns.length;
    }
    clear() {
        this._fns.length = 0;
    }
}
//# sourceMappingURL=Emitter.js.map