export class Value {
    constructor(_value) {
        this._value = _value;
        this.fns = [];
        this.asyncFns = [];
        this.asyncQueued = false;
    }
    on(fn, opts) {
        (opts && opts.async ? this.asyncFns : this.fns).push(fn);
        return () => this.off(fn, opts);
    }
    off(fn, opts) {
        const fns = opts && opts.async ? this.asyncFns : this.fns;
        const i = fns.indexOf(fn);
        if (i >= 0) {
            fns.splice(i, 1);
        }
    }
    once(fn, opts) {
        let off = this.on((data, acc) => {
            const r = fn(data, acc);
            if (off) {
                off();
                off = null;
            }
            return r;
        }, opts);
        return off;
    }
    get value() {
        return this._value;
    }
    get() {
        return this._value;
    }
    set(value) {
        const oldValue = this._value;
        const newValue = typeof value === 'function' ? value(oldValue) : value;
        if (newValue !== oldValue) {
            this._value = newValue;
            const len = this.fns.length;
            if (len > 0) {
                for (let i = 0, fns = this.fns.slice(); i < len; i += 1) {
                    fns[i](newValue, oldValue);
                }
            }
            if (!this.asyncQueued && this.asyncFns.length) {
                this.asyncQueued = true;
                setTimeout(() => {
                    this.asyncQueued = false;
                    const newValueAsync = this._value;
                    if (newValueAsync !== oldValue) {
                        for (let ai = 0, afns = this.asyncFns.slice(), alen = afns.length; ai < alen; ai += 1) {
                            afns[ai](newValueAsync, oldValue);
                        }
                    }
                }, 0);
            }
        }
    }
    clear() {
        this.fns.length = this.asyncFns.length = 0;
    }
}
//# sourceMappingURL=Value.js.map