export class TimeoutError extends Error {
}
export class DeferredPromise {
    constructor(timeout) {
        this.timeout = timeout;
        this._resolve = null;
        this._reject = null;
        this._fulfilled = false;
        this._promise = new Promise((resolve, reject) => {
            this._resolve = resolve;
            this._reject = reject;
        });
        if (timeout) {
            this._tid = setTimeout(() => {
                if (this._reject) {
                    this.reject(new TimeoutError('timeout'));
                }
            }, timeout);
        }
    }
    resolve(value) {
        const { _resolve } = this;
        if (_resolve) {
            this._fulfilled = true;
            this.clear();
            _resolve(value);
        }
    }
    reject(reason) {
        const { _reject } = this;
        if (_reject) {
            this.clear();
            _reject(reason);
        }
    }
    then(onFulfilled, onRejected) {
        return this._promise.then(onFulfilled, onRejected);
    }
    catch(onRejected) {
        return this._promise.catch(onRejected);
    }
    finally(onFinally) {
        return this._promise.then((v) => {
            onFinally();
            return v;
        }, (e) => {
            onFinally();
            throw e;
        });
    }
    clear() {
        this._resolve = null;
        this._reject = null;
        const { _tid } = this;
        if (_tid) {
            clearTimeout(_tid);
            this._tid = undefined;
        }
    }
    get promise() {
        return this._promise;
    }
    get pending() {
        return Boolean(this._reject);
    }
    get fulfilled() {
        return this._fulfilled;
    }
    get rejected() {
        return !(this._fulfilled || this.pending);
    }
    get status() {
        return this.pending ? 'pending' : this.fulfilled ? 'fulfilled' : 'rejected';
    }
}
//# sourceMappingURL=DeferredPromise.js.map