export const createBatcher = (batch, delay = 0) => {
    let argsQueue = [];
    let promise = null;
    const queueAndWait = (args) => {
        argsQueue.push(args);
        if (!promise) {
            promise = new Promise((resolve, reject) => {
                setTimeout(() => {
                    const argsList = argsQueue;
                    argsQueue = [];
                    promise = null;
                    try {
                        batch(argsList).then(resolve, reject);
                    }
                    catch (e) {
                        reject(e);
                    }
                }, delay);
            });
        }
        return promise;
    };
    return (...args) => {
        const index = argsQueue.length;
        return queueAndWait(args).then((res) => res[index], (e) => {
            if (Array.isArray(e)) {
                throw e[index];
            }
            throw e;
        });
    };
};
//# sourceMappingURL=createBatcher.js.map